def linux__magic_area(k):
	""" see the SKF_AD_ macros or bpfc(8) manual page from netsniff-ng """
	skf_ancillary_data = {0: 'protocol', 4: 'pkttype', 8: 'ifindex',
	    12: 'nlattr', 16: 'nlattr_nest', 20: 'mark', 24: 'queue',
	    28: 'hatype', 32: 'rxhash', 36: 'cpu', 40: 'alu_xor_x',
	    44: 'vlan', 48: 'vlan_present', 52: 'poff' }
	if k < 0xFFFFF000 or k > 0xFFFFFFFF:
		return ' [{0}]'.format(k)
	k &= ~0xFFFFF000
	if k in skf_ancillary_data:
		return " #{0}".format(skf_ancillary_data[k])
	return " UNKNOWN EXTENSION {0}".format(k)

BPF_LD	= 0x00
BPF_LDX	= 0x01
BPF_ST	= 0x02
BPF_STX	= 0x03
BPF_ALU	= 0x04
BPF_JMP	= 0x05
BPF_RET	= 0x06
BPF_MISC = 0x07

def insn_class(insn):
	return insn[0] & 0x07

## ld/ldx fields

BPF_W	= 0x00
BPF_H	= 0x08
BPF_B	= 0x10

def insn_size(insn):
	return insn[0] & 0x18
def insn_size_str(insn):
	s = insn_size(insn)
	return ('b' if s == BPF_B else 'h' if s == BPF_H else '')

BPF_IMM	= 0x00
BPF_ABS	= 0x20
BPF_IND	= 0x40
BPF_MEM	= 0x60
BPF_LEN	= 0x80
BPF_MSH	= 0xA0

def insn_mode(insn):
	return insn[0] & 0xE0
def insn_target_str(insn):
	m = insn_mode(insn)
	if m == BPF_IMM:
		return 'i {0}'.format(insn[3])
	elif m == BPF_ABS:
		return linux__magic_area(insn[3])
	elif m == BPF_IND:
		### what if K?
		reg = insn_src_str(insn)
		return ' [%{0} + {1}]'.format(reg, insn[3])
	elif m == BPF_MEM:
		return ' M[{0}]'.format(insn[3])
	elif m == BPF_LEN:
		return ' #len'.format(insn[3])
	elif m == BPF_MSH:
		return ' 4 * ([{0}] & 0xf)'.format(insn[3])
	else:
		return 'UNKNOWN INSN MODE'

## alu/jmp fields

BPF_ADD	= 0x00
BPF_SUB	= 0x10
BPF_MUL	= 0x20
BPF_DIV	= 0x30
BPF_OR	= 0x40
BPF_AND	= 0x50
BPF_LSH	= 0x60
BPF_RSH	= 0x70
BPF_NEG	= 0x80
BPF_MOD	= 0x90
BPF_XOR	= 0xA0

BPF_JA	= 0x00
BPF_JEQ	= 0x10
BPF_JGT	= 0x20
BPF_JGE	= 0x30
BPF_JSET= 0x40

def insn_op(insn):
	return insn[0] & 0xF0

BPF_K	= 0x00
BPF_X	= 0x08
BPF_A	= 0x10

def insn_src(insn):
	return insn[0] & 0x08
def insn_src_str(insn):
	if insn_src(insn) == BPF_K:
		return 'k'
	elif insn_src(insn) == BPF_X:
		return 'x'
	else:
		return 'UNKNOWN REGISTER'

def insn_retval(insn):
	return insn[0] & 0x18

# misc operations

BPF_TAX	= 0x00
BPF_TXA	= 0x80

def insn_miscop(insn):
	return insn[0] & 0xF8


class BPF:
	def dis(insn):
		c = insn_class(insn)
		if c == BPF_LD:
			return (BPF.dis_ld(insn), None)
		elif c == BPF_LDX:
			return (BPF.dis_ldx(insn), None)
		elif c == BPF_ST:
			return (BPF.dis_st(insn), None)
		elif c == BPF_STX:
			return (BPF.dis_stx(insn), None)
		elif c == BPF_ALU:
			return (BPF.dis_alu(insn), None)
		elif c == BPF_JMP:
			return BPF.dis_jmp(insn)
		elif c == BPF_RET:
			return (BPF.dis_ret(insn), None)
		elif c == BPF_MISC:
			return (BPF.dis_misc(insn), None)

	def dis_ld(insn):
		ret = "ld" + insn_size_str(insn) + insn_target_str(insn)
		return ret

	def dis_ldx(insn):
		ret = "ldx" + insn_size_str(insn) + insn_target_str(insn)
		return ret

	def dis_st(insn):
		return "st M[{0}]".format(insn[3])

	def dis_stx(insn):
		return "stx M[{0}]".format(insn[3])

	def dis_alu(insn):
		o = insn_op(insn)
		if o == BPF_ADD:	ret = "add"
		elif o == BPF_SUB:	ret = "sub"
		elif o == BPF_MUL:	ret = "mul"
		elif o == BPF_DIV:	ret = "div"
		elif o == BPF_OR:	ret = "or"
		elif o == BPF_AND:	ret = "and"
		elif o == BPF_LSH:	ret = "lsh"
		elif o == BPF_RSH:	ret = "rsh"
		elif o == BPF_NEG:	return "neg"
		elif o == BPF_MOD:	ret = "mod"
		elif o == BPF_XOR:	ret = "xor"

		if insn_src_str(insn) == 'x':
			return ret + ' x'
		else:
			return ret + ' #{0}'.format(insn[3])

	def dis_jmp(insn):
		o = insn_op(insn)
		if o == BPF_JA:		ret = "ja"
		elif o == BPF_JEQ:	ret = "jeq" if insn[1] != 0 else "jne"
		elif o == BPF_JGT:	ret = "jgt" if insn[1] != 0 else "jle"
		elif o == BPF_JGE:	ret = "jge" if insn[1] != 0 else "jlt"
		elif o == BPF_JSET:	ret = "jset"

		ret += " #0x{0:x}".format(insn[3])
		if insn[1] != 0:
			return ret, insn[1]
		else:
			return ret, insn[2]

	def dis_ret(insn):
		rval = insn_retval(insn)
		if rval == BPF_A:	return 'ret a'
		elif rval == BPF_K:	return 'ret #0x{0:x}'.format(insn[3])
		# this one isn't supported according to bpfc(8)
		elif rval == BPF_X:	return 'ret M[{0}]  ???'.format(insn[3])
		return "UNKNOWN RET OP"

	def dis_misc(insn):
		m = insn_miscop(insn)
		return 'tax' if m == BPF_TAX else 'txa' if m == BPF_TXA else 'UNKNOWN MISC OP'


