import re
hfsc_param_re = re.compile('(realtime|linkshare|upperlimit)(\(([0-9.]+[KMG ]b) ([0-9]+) ([0-9.]+[KMG ]b)\)| [0-9.]+[KMG ]b)(.*)')

from parsers.tc import hfsc_d

def ffs(x):
	cnt = 0
	while not (x & 1):
		x >>= 1
		cnt += 1
	return cnt

def bandwidth(s):
	s = s.strip()
	deduct = 2 if s.endswith('b') else 1	# Kb vs. K

	if s.endswith('Gb') or s.endswith('G'):
		f = float(s[:-deduct]) * 1000 * 1000 * 1000
	elif s.endswith('Mb') or s.endswith('M'):
		f = float(s[:-deduct]) * 1000 * 1000
	elif s.endswith('Kb') or s.endswith('K'):
		f = float(s[:-deduct]) * 1000
	elif s.endswith(' b'):
		f = int(s[:-deduct])
	else:
		f = int(s)
	return int(f)

def BSDParseLinks(s):
	allifs = dict()
	for line in s.split(b'\n'):
		if not line:
			continue
		elif line[0] in [ord(' '), ord('\t')]:
			a = line.split(b':')
			key = a[0].strip().decode('ascii')
			if key.startswith('inet6'):
				in6 = line.split()
				if in6[1].startswith(b'fe80::'):
					lla = in6[1] + b'/' + in6[3]
					last['ip6lladdr'] = lla.decode('ascii')
				else:
					inet6 = in6[1] + b'/' + in6[3]
					last['ip6addr'] = inet6.decode('ascii')
			elif key.startswith('inet'):
				in4 = line.split()
				octets = map(lambda x: int(x.decode('ascii')), in4[1].split(b'.'))
				addr = '.'.join(map(str, octets))
				plen = 32 - ffs(int(in4[3], 16))
				last['ip4addr'] = "{0}/{1}".format(addr, plen)
				continue
			elif key.startswith('lladdr'):
				last['lladdr'] = line.split()[1]
			elif key not in ['ifname', 'flags', 'mtu', 'rdomain']:
				last[key] = b':'.join(a[1:]).strip()
		else:
			a = line.split()
			last = dict()

			ifname = a[0][:-1].decode('ascii')
			last['ifname'] = ifname
			last['flags'] = b'<' + a[1].split(b'<')[-1]
			if len(a) >= 3 and a[2] == b'mtu':
				last['mtu'] = int(a[3])

			allifs[ifname] = last
	return allifs

def BSDParseNewQueue(s):
	def parse_curve(split):
		m1 = d = 0
		m2 = split[0][:-1] if split[0][-1] == ord(',') else split[0]
		m2 = bandwidth(m2.decode('ascii'))
		off = 1
		if split[off] == b'burst':
			m1 = bandwidth(split[off+1].decode('ascii'))
			off += 2
		if split[off] == b'for':
			d = split[off+1]
			if split[off+1][-1] == ord(','):
				d = split[off+1][:-1]
			d = hfsc_d(d.decode('ascii'))
			off += 2
		return ((m1, d, m2), split[off:])

	queues = dict()
	for line in s.split(b'\n'):
		if not line or not line.startswith(b'queue '):
			continue
		a = line.split()
		q = dict()
		q['kids'] = []
		qname = a[1].decode('ascii')
		parent = None

		off = 2
		if a[off] == b'parent':
			parent = a[off+1].decode('ascii')
			q['parent'] = parent
			off += 2

		if a[off] == b'on':
			q['dev'] = a[off+1].decode('ascii')
			if parent:
				queues[q['dev']][parent]['kids'] += [qname]
			off += 2

		if a[off] != b'bandwidth':
			raise Exception('bad line >>{0}<<'.format(line))
		q['linkshare'], a = parse_curve(a[off+1:])
		off = 0

		# parse the HFSC curves
		while a[off] not in [b'default', b'qlimit']:
			if a[off] == b'max':
				q['upperlimit'], a = parse_curve(a[off+1:])
				off = 0
			elif a[off] == b'min':
				q['realtime'], a = parse_curve(a[off+1:])
				off = 0
			else:
				raise Exception('not a curve >>{0}<<'.format(a[off]))

		# other options at the end
		if a[off] == b'default':
			q['default'] = True
			off += 1
		if a[off] == b'qlimit':
			q['qlimit'] = int(a[off+1])
		q['name'] = qname

		if q['dev'] not in queues:
			queues[q['dev']] = dict()
		queues[q['dev']][qname] = q
	return queues

def BSDParseALTQ(s):
	altqs = dict()
	for line in s.split(b'\n'):
		if not line:
			continue
		a = line.split()
		if a[0] != b'queue': # such lines shouldn't ever appear
			if not line.startswith(b"No queue in use"):
				print(line)
			continue
		q = dict()
		qname = a[1].decode('ascii')
		off = 2
		if a[off] == b'on':
			q['dev'] = a[off+1].decode('ascii')
			off += 2
		if a[off] == b'bandwidth':
			q['bandwidth'] = bandwidth(a[off+1].decode('ascii'))
			# either "256Kb" or "256 b"
			if len(a) > off + 2 and a[off+2] == 'b':
				off += 3
			else:
				off += 2
		if a[off] == b'priority':
			q['priority'] = int(a[off+1])
			off += 2
		if a[off] == b'qlimit':
			q['qlimit'] = int(a[off+1])
			off += 2
		### XXX install OpenBSD 5.5 and try out oldqueue/queue
		kids_start = line.find(b'{')
		if a[off][0] == '{': # root queues don't have disciplines
			q['disc'] = "root"
			if line[-1] != ord('}'):
				raise Exception("wrong line: {0}".format(line))
			kids = line[(kids_start + 1):-1].split(b',')
			q['kids'] = [x.strip().decode('ascii') for x in kids]
			q['params'] = "none"
		else:
			if kids_start != -1:
				# "hfsc( upperlimit() )" is weird...
				q['disc'] = a[off][:-1].decode('ascii')
				if line[-1] != ord('}'):
					raise Exception("wrong line: {0}".format(line))
				kids = line[kids_start+1:-1].split(b',')
				q['kids'] = [x.strip().decode('ascii') for x in kids]
				# first wipe kids, then extract params
				tmp = line[:kids_start].split()
				q['params'] = '(' + ' '.join(map(lambda x: x.decode('ascii'), tmp[off+1:]))
			else:
				q['disc'] = a[off][:-1].decode('ascii')
				q['params'] = '(' + ' '.join(map(lambda x: x.decode('ascii'), a[off+1:]))
				q['kids'] = []
		q['name'] = qname
		if qname == 'root_' + q['dev']:
			altqs[q['dev']] = dict()
		altqs[q['dev']][qname] = q
	return altqs

def BSDParseHFSC(s):
	if s[0] != '(' or s[-1] != ')':
		raise Exception("hfsc bad param: >>{0}<<".format(s))
	s = s[1:-1]

	curves = dict()
	for key in ['default', 'red', 'ecn']:
		curves[key] = False

	if s.startswith('red'):
		curves['red'] = True
		s = s[3:].strip()
	if s.startswith('ecn'):
		curves['ecn'] = True
		s = s[3:].strip()
	if s.startswith('default'):
		curves['default'] = True
		s = s[7:].strip()

	while s:
		m = hfsc_param_re.match(s)
		if not m:
			raise Exception("hfsc bad curves: {0}".format(s))

		g = m.groups()
		curve = g[0]
		if g[1][0] == '(':
			(m1, m2) = map(bandwidth, (g[2], g[4]))
			d = int(g[3])
		else:
			m2 = bandwidth(g[1])
			(m1, d) = (0, 0)
		curves[curve] = (m1, d, m2)
		s = g[len(g)-1].strip()

	return curves

