import fix_path
import unittest

from parsers.tc import TcParseBPF

## ============================================================= ><8 ----------
class test_bpf(unittest.TestCase):
	def setUp(self):
		self.maxDiff = 4096

	def test_load_jump_ret(self):
		input = b'4,40 0 0 12,21 0 1 2048,6 0 0 4294967295,6 0 0 0'
		ret = TcParseBPF(input)
		self.assertEqual(ret[0], "ldh [12]	(40, 0, 0, 12)")
		self.assertEqual(ret[1], "jne #0x800, L3	(21, 0, 1, 2048)")
		self.assertEqual(ret[2], "ret #0xffffffff	(6, 0, 0, 4294967295)")
		self.assertEqual(ret[3], "L3:\tret #0x0	(6, 0, 0, 0)")

	def test_loads_var_len(self):
		input = b'10,40 0 0 12,21 0 7 34525,48 0 0 20,21 0 5 6,40 0 0 54,21 2 0 22,40 0 0 56,21 0 1 22,6 0 0 65535,6 0 0 0'
		ret = TcParseBPF(input)
		self.assertRegex(ret[0], "^ldh \[12\]")
		self.assertRegex(ret[1], "^jne #0x86dd, L9")
		self.assertRegex(ret[2], "^ldb \[20\]")
		self.assertRegex(ret[3], "^jne #0x6, L9")
		self.assertRegex(ret[4], "^ldh \[54\]")
		self.assertRegex(ret[5], "^jeq #0x16, L8")
		self.assertRegex(ret[6], "^ldh \[56\]")
		self.assertRegex(ret[7], "^jne #0x16, L9")
		self.assertRegex(ret[8], "^L8:\tret #0xffff")
		self.assertRegex(ret[9], "^L9:\tret #0x0")

	def test_ssh_ipv4(self):
		input = b'13,40 0 0 12,21 0 10 2048,48 0 0 23,21 0 8 6,40 0 0 20,69 6 0 8191,177 0 0 14,72 0 0 14,21 2 0 22,72 0 0 16,21 0 1 22,6 0 0 4294967295,6 0 0 0'
		ret = TcParseBPF(input)
		self.assertRegex(ret[0], "^ldh \[12\]")
		self.assertRegex(ret[1], "^jne #0x800, L12")
		self.assertRegex(ret[2], "^ldb \[23\]")
		self.assertRegex(ret[3], "^jne #0x6, L12")
		self.assertRegex(ret[4], "^ldh \[20\]")
		self.assertRegex(ret[5], "^jset #0x1fff, L12")
		self.assertRegex(ret[6], "^ldxb 4 \* \(\[14\] & 0xf\)")
		self.assertRegex(ret[7], "^ldh \[%x \+ 14\]")
		self.assertRegex(ret[8], "^jeq #0x16, L11")
		self.assertRegex(ret[9], "^ldh \[%x \+ 16\]")
		self.assertRegex(ret[10], "^jne #0x16, L12")
		self.assertRegex(ret[11], "^L11:\tret #0xffffffff")
		self.assertRegex(ret[12], "^L12:\tret #0x0")

	def test_stores_arith_specials(self):
		input = b'21,1 0 0 8,3 0 0 3,1 0 0 12,3 0 0 4,96 0 0 3,2 0 0 1,53 13 0 42,12 0 0 0,20 0 0 42,44 0 0 0,53 0 9 42,116 0 0 2,132 0 0 0,37 0 6 32,7 0 0 0,135 0 0 0,32 0 0 4294963220,129 0 0 0,2 0 0 15,3 0 0 5,22 0 0 0'
		ret = TcParseBPF(input)
		self.assertRegex(ret[0], "^ldxi 8")
		self.assertRegex(ret[1], "^stx M\[3\]")
		self.assertRegex(ret[2], "^ldxi 12")
		self.assertRegex(ret[3], "^stx M\[4\]")
		self.assertRegex(ret[4], "^ld M\[3\]")
		self.assertRegex(ret[5], "^st M\[1\]")
		self.assertRegex(ret[6], "^jge #0x2a, L20")
		self.assertRegex(ret[7], "^add x")
		self.assertRegex(ret[8], "^sub #42")
		self.assertRegex(ret[9], "^mul x")
		self.assertRegex(ret[10], "^jlt #0x2a, L20")
		self.assertRegex(ret[11], "^rsh #2")
		self.assertRegex(ret[12], "^neg\t")
		self.assertRegex(ret[13], "^jle #0x20, L20")
		self.assertRegex(ret[14], "^tax")
		self.assertRegex(ret[15], "^txa")
		self.assertRegex(ret[16], "^ld #mark")
		self.assertRegex(ret[17], "^ldx #len")
		self.assertRegex(ret[18], "^st M\[15\]")
		self.assertRegex(ret[19], "^stx M\[5\]")
		self.assertRegex(ret[20], "^L20:\tret a\t")

if __name__ == '__main__':
	unittest.main()

