import fix_path
import unittest

from parsers.bsd import BSDParseALTQ, BSDParseNewQueue

##
NEWQUEUE_inputs = []
NEWQUEUE_expect = []

##
## NEWQUEUE
##
NEWQUEUE_inputs.append(b'''
queue root on em2 bandwidth 990M qlimit 50
queue misc parent root on em2 bandwidth 20M, max 200M default qlimit 50
queue 584_sedla parent root on em2 bandwidth 65K, max 26M burst 31M for 10000ms qlimit 50
queue 783_lukes parent root on em2 bandwidth 65K, max 8M burst 10M for 10000ms qlimit 50
queue 898_novak parent root on em2 bandwidth 65K, max 15M burst 20M for 10000ms qlimit 50
queue 466_grego parent root on em2 bandwidth 65K, max 52M burst 62M for 10000ms qlimit 50
queue 1133_svato parent root on em2 bandwidth 65K, max 8M burst 10M for 10000ms qlimit 50
queue lowdelay parent root on em2 bandwidth 4M burst 30M for 1000ms qlimit 700
queue servery parent root on em2 bandwidth 100M, max 990M qlimit 250
queue pater parent root on em2 bandwidth 6M, max 6M qlimit 50
''')
NEWQUEUE_expect.append(lambda test, a:
	test.assertEqual(a['em2']['root']['linkshare'],	(0, 0, 990000000)) or
	test.assertEqual(a['em2']['root']['qlimit'],	50) or
	test.assertEqual(a['em2']['root']['dev'],	'em2') or
	test.assertEqual(a['em2']['584_sedla']['upperlimit'],	(31000000, 10000000, 26000000)) or
	test.assertEqual(a['em2']['584_sedla']['linkshare'],	(0, 0, 65000)) or
	test.assertEqual(a['em2']['584_sedla']['qlimit'],	50) or

	test.assertEqual(a['em2']['lowdelay']['linkshare'],	(30000000, 1000000, 4000000)) or
	test.assertEqual(a['em2']['lowdelay']['qlimit'],	700) or

	test.assertEqual(a['em2']['pater']['upperlimit'],	(0, 0, 6000000)) or
	test.assertEqual(a['em2']['pater']['linkshare'],	(0, 0, 6000000)) or
	test.assertEqual(a['em2']['pater']['qlimit'],	50) or
	True
)

##
## NEWQUEUE
##
NEWQUEUE_inputs.append(b'''
queue root on vmx0 bandwidth 300M qlimit 50
queue lowdelay parent root on vmx0 bandwidth 4M, min 5M burst 40M for 500ms, max 14M qlimit 200
queue servery parent root on vmx0 bandwidth 10M, max 200M qlimit 50
queue misc parent root on vmx0 bandwidth 64K, max 2M default qlimit 50
queue zakaznik1 parent root on vmx0 bandwidth 1M burst 3M for 5000ms, max 20M qlimit 50
queue z1a parent zakaznik1 on vmx0 bandwidth 64K, min 512K, max 1M qlimit 50
queue z1b parent zakaznik1 on vmx0 bandwidth 64K, max 20M qlimit 50
queue zakaznik2 parent root on vmx0 bandwidth 2M burst 6M for 5000ms, min 512K, max 30M qlimit 50
''')
NEWQUEUE_expect.append(lambda test, a:
	test.assertEqual(a['vmx0']['root']['linkshare'],	(0, 0, 300000000)) or
	test.assertEqual(a['vmx0']['root']['qlimit'],	50) or

	test.assertEqual(a['vmx0']['lowdelay']['upperlimit'],	(0, 0, 14000000)) or
	test.assertEqual(a['vmx0']['lowdelay']['linkshare'],	(0, 0, 4000000)) or
	test.assertEqual(a['vmx0']['lowdelay']['realtime'],	(40000000, 500000, 5000000)) or
	test.assertEqual(a['vmx0']['lowdelay']['qlimit'],	200) or

	test.assertEqual(a['vmx0']['z1a']['upperlimit'],	(0, 0, 1000000)) or
	test.assertEqual(a['vmx0']['z1a']['realtime'],		(0, 0, 512000)) or
	test.assertEqual(a['vmx0']['z1a']['linkshare'],		(0, 0, 64000)) or
	test.assertEqual(a['vmx0']['z1a']['parent'],	'zakaznik1') or

	test.assertEqual(a['vmx0']['zakaznik2']['upperlimit'],	(0, 0, 30000000)) or
	test.assertEqual(a['vmx0']['zakaznik2']['linkshare'],	(6000000, 5000000, 2000000)) or
	test.assertEqual(a['vmx0']['zakaznik2']['realtime'],	(0, 0, 512000)) or
	test.assertEqual(a['vmx0']['zakaznik2']['qlimit'],	50) or
	test.assertEqual(a['vmx0']['zakaznik2']['parent'],	'root') or

	test.assertEqual(a['vmx0']['misc']['upperlimit'],	(0, 0, 2000000)) or
	test.assertEqual(a['vmx0']['misc']['linkshare'],	(0, 0, 64000)) or
	test.assertEqual(a['vmx0']['misc']['default'],	True) or
	test.assertEqual(a['vmx0']['misc']['qlimit'],	50) or
	True
)

## ============================================================= ><8 ----------
class test_filters(unittest.TestCase):
	def setUp(self):
		self.maxDiff = 4096

	def test_all_inputs(self):
		for x in range(len(NEWQUEUE_inputs)):
			a = BSDParseNewQueue(NEWQUEUE_inputs[x])
			self.assertTrue(NEWQUEUE_expect[x](self, a))

if __name__ == '__main__':
	unittest.main()

