import fix_path
import unittest

from parsers.tc import TcParseEmatch, TcParseFilters

##
## Tests parsed outputs of "tc filter show dev enp1s0f1 parent FFFF:"
##
TC_FILTER_SHOW_inputs = []
TC_FILTER_SHOW_expect = []

##
## INGRESS QDISC REDIRECTING TO ifb0
## tc qdisc add dev enp1s0f1 ingress
## tc filter add dev enp1s0f1 parent FFFF: basic action mirred egress redirect dev ifb0
## tc qdisc add dev ifb0 root hhf
##
TC_FILTER_SHOW_inputs.append(b'''
filter protocol all pref 49152 basic 
filter protocol all pref 49152 basic handle 0x1 
	action order 1: mirred (Egress Redirect to device ifb0) stolen
 	index 1 ref 1 bind 1
''')
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[49152].actions,	[]) or
	test.assertEqual(a[49152].matches,	[]) or
	test.assertEqual(a[49152].handle,	None) or
	test.assertEqual(a[49152].type,		'basic') or
	test.assertEqual(a[49152].children[0x1].handle,		0x1) or
	test.assertEqual(a[49152].children[0x1].matches,	[]) or
	test.assertEqual(a[49152].children[0x1].type,		'basic') or
	test.assertEqual(a[49152].children[0x1].actions[0].order,	1) or
	test.assertEqual(a[49152].children[0x1].actions[0].dev,	'ifb0') or
	test.assertEqual(a[49152].children[0x1].actions[0].direction,	'egress') or
	test.assertEqual(a[49152].children[0x1].actions[0].state,	'action') or
	test.assertEqual(a[49152].children[0x1].actions[0].type,	'redirect') or
	test.assertEqual(a[49152].children[0x1].actions[0].action.type, 'stolen') or
	True
)

##
## U32 (note that in practice you need "protocol ip" here)
## tc filter add dev enp1s0f1 parent FFFF: u32 match ip src 10.0.0.4 match ip dport 5001 0xFFFF classid FFFF:BAC
## 
TC_FILTER_SHOW_inputs.append(b'''
filter protocol all pref 49152 u32 
filter protocol all pref 49152 u32 fh 800: ht divisor 1 
filter protocol all pref 49152 u32 fh 800::800 order 2048 key ht 800 bkt 0 flowid ffff:bac 
  match 0a000004/ffffffff at 12
  match 00001389/0000ffff at 20
''')
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[49152].actions,	[]) or
	test.assertEqual(a[49152].matches,	[]) or
	test.assertEqual(a[49152].parent,	None) or
	test.assertEqual(a[49152].children[2048].htid,		2048) or
	test.assertEqual(a[49152].children[2048].bkt,		0) or
	test.assertEqual(a[49152].children[2048].key,		0) or
	test.assertEqual(a[49152].children[2048].proto,		'all') or
	test.assertEqual(a[49152].children[2048].pref,		49152) or
	test.assertEqual(a[49152].children[2048].type,		'u32') or
	test.assertEqual(a[49152].children[2048].fh,		b'800:') or
	test.assertEqual(a[49152].children[2048].divisor,	1) or
	test.assertEqual(a[49152].children[2048].children[2048].htid,	2048) or
	test.assertEqual(a[49152].children[2048].children[2048].bkt,	0) or
	test.assertEqual(a[49152].children[2048].children[2048].key,	2048) or
	test.assertEqual(a[49152].children[2048].children[2048].parent,	None) or
	test.assertEqual(a[49152].children[2048].children[2048].proto,	'all') or
	test.assertEqual(a[49152].children[2048].children[2048].pref,	49152) or
	test.assertEqual(a[49152].children[2048].children[2048].type,	'u32') or
	test.assertEqual(a[49152].children[2048].children[2048].classid,	'ffff:bac') or
	test.assertEqual(a[49152].children[2048].children[2048].matches[0],	(0x0A000004, 0xFFFFFFFF, 12)) or
	test.assertEqual(a[49152].children[2048].children[2048].matches[1],	(5001, 0xFFFF, 20)) or
	True
)

##
## U32 HASH TABLE WITH TWO BUCKETS (not specifying pref explicitly will create bogus empty tables)
##
## tc filter del dev enp1s0f1 protocol ip parent FFFF: pref 42 u32
## 
## echo "divisor"
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: handle A: u32 divisor 256
## 
## echo "link"
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 link A: hashkey mask 0x0000FF00 at 12 match ip src 10.144.0.0/16
##
## echo "nuly"
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A: match ip src 10.144.0.1 match ip protocol 6 0xFF classid FF00:0601
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A: match ip src 10.144.0.1 match ip protocol 17 0xFF classid FF00:1101
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A: match ip src 10.144.0.2 match ip protocol 6 0xFF classid FF00:0602
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A: match ip src 10.144.0.2 match ip protocol 17 0xFF classid FF00:1102
## 
## echo "trojky"
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A:3: match ip src 10.144.3.8 match ip protocol 6 0xFF classid FF03:0608
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A:3: match ip src 10.144.3.8 match ip protocol 17 0xFF classid FF03:1108
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A:3: match ip src 10.144.3.9 match ip protocol 6 0xFF classid FF03:0609
## tc filter add dev enp1s0f1 protocol ip pref 42 parent FFFF: u32 ht A:3: match ip src 10.144.3.9 match ip protocol 17 0xFF classid FF03:1109
## 
TC_FILTER_SHOW_inputs.append(b'''
filter protocol ip pref 42 u32 
filter protocol ip pref 42 u32 fh a: ht divisor 256 
filter protocol ip pref 42 u32 fh a::800 order 2048 key ht a bkt 0 flowid ff00:601 
  match 0a900001/ffffffff at 12
  match 00060000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a::801 order 2049 key ht a bkt 0 flowid ff00:1101 
  match 0a900001/ffffffff at 12
  match 00110000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a::802 order 2050 key ht a bkt 0 flowid ff00:602 
  match 0a900002/ffffffff at 12
  match 00060000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a::803 order 2051 key ht a bkt 0 flowid ff00:1102 
  match 0a900002/ffffffff at 12
  match 00110000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a:3:800 order 2048 key ht a bkt 3 flowid ff03:608 
  match 0a900308/ffffffff at 12
  match 00060000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a:3:801 order 2049 key ht a bkt 3 flowid ff03:1108 
  match 0a900308/ffffffff at 12
  match 00110000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a:3:802 order 2050 key ht a bkt 3 flowid ff03:609 
  match 0a900309/ffffffff at 12
  match 00060000/00ff0000 at 8
filter protocol ip pref 42 u32 fh a:3:803 order 2051 key ht a bkt 3 flowid ff03:1109 
  match 0a900309/ffffffff at 12
  match 00110000/00ff0000 at 8
filter protocol ip pref 42 u32 fh 800: ht divisor 1 
filter protocol ip pref 42 u32 fh 800::800 order 2048 key ht 800 bkt 0 link a: 
  match 0a900000/ffff0000 at 12
    hash mask 0000ff00 at 12 
''')
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[42].actions,	[]) or
	test.assertEqual(a[42].matches,	[]) or
	test.assertEqual(a[42].parent,	None) or
	## this time table 800:: is only there for the "link a:" with hashkey+off+mask+match
	test.assertEqual(a[42].children[2048].htid,	2048) or
	test.assertEqual(a[42].children[2048].bkt,	0) or
	test.assertEqual(a[42].children[2048].key,	0) or
	test.assertEqual(a[42].children[2048].proto,	'ip') or
	test.assertEqual(a[42].children[2048].pref,	42) or
	test.assertEqual(a[42].children[2048].type,	'u32') or
	test.assertEqual(a[42].children[2048].fh,	b'800:') or
	test.assertEqual(a[42].children[2048].divisor,	1) or
	# this rule does the "shootout" to the A:: table for 10.144.0.0/16 based on its third octet
	test.assertEqual(a[42].children[2048].children[2048].bkt,	0) or
	test.assertEqual(a[42].children[2048].children[2048].key,	2048) or
	test.assertEqual(a[42].children[2048].children[2048].link,	'a:0') or
	test.assertEqual(a[42].children[2048].children[2048].hashmask,	0x0000FF00) or
	test.assertEqual(a[42].children[2048].children[2048].hashoff,	12) or
	test.assertEqual(a[42].children[2048].children[2048].children,	{}) or
	test.assertEqual(a[42].children[2048].children[2048].matches[0],(0x0A900000, 0xFFFF0000, 12)) or
	test.assertEqual(str(a[42].children[2048].children[2048]),	'fh 800::800 link a:0 hash mask 0000ff00 at 12') or
	# the hash table where the traffic goes
	test.assertEqual(a[42].children[0xA].divisor,	256) or
	test.assertEqual(a[42].children[0xA].htid,	0xA) or
	test.assertEqual(str(a[42].children[0xA]),	'fh a: divisor 256') or
	# zeroth bucket, first entry
	test.assertEqual(a[42].children[0xA].children[2048].htid,	10) or
	test.assertEqual(a[42].children[0xA].children[2048].bkt,	0) or
	test.assertEqual(a[42].children[0xA].children[2048].key,	2048) or
	test.assertEqual(a[42].children[0xA].children[2048].parent,	None) or
	test.assertEqual(a[42].children[0xA].children[2048].classid,	'ff00:601') or
	# TCP from 10.144.0.1
	test.assertEqual(a[42].children[0xA].children[2048].matches[0],	(0x0A900001, 0xFFFFFFFF, 12)) or
	test.assertEqual(a[42].children[0xA].children[2048].matches[1],	(6 << 16, 0xFF0000, 8)) or
	# third bucket (bucket numbers are shifted by 12, sizeof(key) in fh), third entry
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].htid,	10) or
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].bkt,	3) or
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].key,	2051) or
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].parent,	None) or
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].classid,	'ff03:1109') or
	test.assertEqual(str(a[42].children[0xA].children[(3<<12)|2051]),	'fh a:3:803 classid ff03:1109') or
	# UDP from 10.144.3.9
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].matches[0],	(0x0A900309, 0xFFFFFFFF, 12)) or
	test.assertEqual(a[42].children[0xA].children[(3<<12)|2051].matches[1],	(17 << 16, 0xFF0000, 8)) or
	True
)

##
## U32 HASH TABLE WITH VARIABLE OFFSETS
##
## tc filter del dev enp1s0f1 protocol ip pref 43 parent FFFF: u32
##
## tc filter add dev enp1s0f1 protocol ip pref 43 parent FFFF: handle B: u32 divisor 256
## tc filter add dev enp1s0f1 protocol ip pref 43 parent FFFF: u32 link B: offset at 32 mask 0xCF00 shift 10 plus 3 eat match ip protocol 55 0xFF
## tc filter add dev enp1s0f1 protocol ip pref 43 parent FFFF: u32 ht B: match ip src 192.168.144.100 match tcp dst 5001 0xFFFF classid DEAD:beef
##
TC_FILTER_SHOW_inputs.append(b'''
filter protocol ip pref 43 u32 
filter protocol ip pref 43 u32 fh b: ht divisor 256 
filter protocol ip pref 43 u32 fh b::800 order 2048 key ht b bkt 0 flowid dead:beef 
  match c0a89064/ffffffff at 12
  match 00001389/0000ffff at nexthdr+0
filter protocol ip pref 43 u32 fh 800: ht divisor 1 
filter protocol ip pref 43 u32 fh 800::800 order 2048 key ht 800 bkt 0 link b: 
  match 00370000/00ff0000 at 8
    offset cf00>>10 at 32 plus 3  eat 
''')
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[43].actions,	[]) or
	test.assertEqual(a[43].matches,	[]) or
	test.assertEqual(a[43].children[2048].children[2048].link,	'b:0') or
	test.assertEqual(a[43].children[2048].children[2048].mask,	0xCF00) or
	test.assertEqual(a[43].children[2048].children[2048].shift,	10) or
	test.assertEqual(a[43].children[2048].children[2048].offoff,	32) or
	test.assertEqual(a[43].children[2048].children[2048].plus,	3) or
	test.assertEqual(a[43].children[2048].children[2048].eat,	True) or
	test.assertEqual(a[43].children[2048].children[2048].matches[0],(0x370000, 0xFF0000, 8)) or
	test.assertEqual(str(a[43].children[2048].children[2048]),	'fh 800::800 link b:0 mask 0000cf00 >> 10 at 32 plus 3 eat') or
	test.assertEqual(a[43].children[0xB].divisor,	256) or
	test.assertEqual(a[43].children[0xB].children[2048].htid,	0xB) or
	test.assertEqual(a[43].children[0xB].children[2048].matches[0],	(0xC0A89064, 0xFFFFFFFF, 12)) or
	test.assertEqual(a[43].children[0xB].children[2048].matches[1],	(5001, 0xFFFF, (1<<16))) or
	test.assertEqual(a[43].children[0xB].children[2048].classid,	'dead:beef') or
	test.assertEqual(str(a[43].children[0xB].children[2048]),	'fh b::800 classid dead:beef') or
	True
)

##
## BASIC FILTERS WITH SOME ACTIONS AND EMATCHES
##
## tc filter del dev enp1s0f1 protocol all pref 44 parent FFFF:
##
## tc filter add dev enp1s0f1 protocol all pref 44 parent FFFF: basic classid C:AAAA match 'meta(loadavg_5 gt 4) or (nbyte(potazmo\r\n""\xFF at 64 layer transport) and not cmp(u16 at 16 trans gt 49152))' action drop
## tc filter add dev enp1s0f1 protocol all pref 44 parent FFFF: basic classid C:BBBB match 'meta(446 eq vlan)' action police rate 25Mbit burst 42 conform-exceed pipe/pass skbedit mark 0xBAD mirred egress mirror dev ifb1
##
## tc filter add dev enp1s0f1 protocol all pref 44 parent FFFF: basic classid C:CCCC match 'cmp(u32 at 12 mask 0xFFFF0000 eq 0x0A0A0000)' action nat egress 10.144.144.0/24 192.168.144.0
## tc filter add dev enp1s0f1 protocol all pref 44 parent FFFF: basic classid C:DDDD match 'u32(u32 0x0A0A0000 0xFFFF0000 at 12)' action skbedit queue_mapping 1
## tc filter add dev enp1s0f1 protocol all pref 44 parent FFFF: basic classid C:EEEE match 'meta(nf_mark shift 4 mask 0xFF00 gt 240)' action pedit munge offset 12 u32 invert munge offset -8 u16 set 0x4141 csum ip4h and tcp or udp
##
##  THE OUTPUT NEEDS ESCAPING OF THAT nbyte STRING!
##
TC_FILTER_SHOW_inputs.append(b'''
filter protocol all pref 44 basic 
filter protocol all pref 44 basic handle 0x5 flowid c:eeee 
  meta(nf_mark shift 4 mask 0x0000ff00 gt 240)

	action order 1:  pedit action pass keys 2
 	 index 26 ref 1 bind 1
	 key #0  at 12: val ffffffff mask ffffffff
	 key #1  at -8: val 41410000 mask 0000ffff
 
	action order 2: csum (iph, tcp, udp) action pass
	index 26 ref 1 bind 1

filter protocol all pref 44 basic handle 0x4 flowid c:dddd 
  u32(0a0a0000/ffff0000 at 12)

	action order 1:  skbedit queue_mapping 1
	 index 52 ref 1 bind 1
 
filter protocol all pref 44 basic handle 0x3 flowid c:cccc 
  cmp(u32 at 12 layer 1 mask 0xffff0000 eq 168427520)

	action order 1:  nat egress 10.144.144.0/24 192.168.144.0 pass
filter protocol all pref 44 basic handle 0x2 flowid c:bbbb 
  meta(446 eq vlan mask 0x00000000)

	action order 1:  police 0x1a rate 25Mbit burst 37b mtu 2Kb action pipe overhead 0b 
ref 1 bind 1

	action order 2:  skbedit mark 2989
	 index 51 ref 1 bind 1
 
	action order 3: mirred (Egress Mirror to device ifb1) pipe
 	index 26 ref 1 bind 1
 
filter protocol all pref 44 basic handle 0x1 flowid c:aaaa 
  meta(loadavg_5 mask 0x00000000 gt 4)
  OR (
    nbyte(70 6f 74 61 7a 6d 6f 5c 72 5c 6e 22 22 5c 78 46 46 "potazmo\\r\\n""\\xFF" at 64 layer 2)
    AND NOT cmp(u16 at 16 layer 1 trans gt 49152)
  ) 
	action order 1: gact action drop
	 random type none pass val 0
	 index 26 ref 1 bind 1
 
''')
__needle__ = list(map(ord, ['p', 'o', 't', 'a', 'z', 'm', 'o', '\\', 'r', '\\', 'n', '"', '"', '\\', 'x', 'F', 'F']))
__nbyte__ = { 'type': 'nbyte', 'layer': 2, 'needle': __needle__, 'offset': 64 }
__inv_cmp__ = { 'type': 'cmp', 'inverted': True, 'layer': 1, 'align': 'u16', 'mask': False, 'offset': 16, 'op': 'gt', 'trans': True, 'value': 49152 }
__nbyte_andnot_cmp__ = { 'left': __nbyte__, 'right': __inv_cmp__, 'type': 'and' }
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[44].actions,	[]) or
	test.assertEqual(a[44].matches,	[]) or
	test.assertEqual(a[44].handle,	None) or
	test.assertEqual(a[44].type,	'basic') or
	# First one.
	test.assertEqual(a[44].children[0x1].classid,	'c:aaaa') or
	test.assertEqual(a[44].children[0x1].type,	'basic') or
	test.assertEqual(str(a[44].children[0x1]),	'handle 0x1 classid c:aaaa') or
	test.assertEqual(a[44].children[0x1].actions[0].type,	'drop') or
	test.assertDictEqual(TcParseEmatch(b' '.join(a[44].children[0x1].ematch))[0],
		{'left': {'type': 'meta', 'rval': 4, 'op': 'gt', 'lval': ('loadavg_5', 0, 0)},
		'right': __nbyte_andnot_cmp__, 'type': 'or'}) or
	# Second one.
	test.assertEqual(a[44].children[0x2].classid,	'c:bbbb') or
	test.assertEqual(a[44].children[0x2].type,	'basic') or
	test.assertEqual(a[44].children[0x2].actions[0].type,		'police') or
	test.assertEqual(a[44].children[0x2].actions[0].rate,		25000000) or
	test.assertEqual(a[44].children[0x2].actions[0].burst,		37) or
	test.assertEqual(a[44].children[0x2].actions[0].mtu,		2000) or
	test.assertEqual(a[44].children[0x2].actions[0].action.type,	'pipe') or
	test.assertEqual(str(a[44].children[0x2].actions[0]),		'police rate 25000000.0 burst 37.0 -> pipe') or
	test.assertEqual(a[44].children[0x2].actions[1].type,		'skbedit') or
	test.assertEqual(a[44].children[0x2].actions[1].mark,		2989) or
	test.assertEqual(str(a[44].children[0x2].actions[1]),		'skbedit mark: 2989') or
	test.assertEqual(a[44].children[0x2].actions[2].type,		'mirror') or
	test.assertEqual(a[44].children[0x2].actions[2].direction,	'egress') or
	test.assertEqual(a[44].children[0x2].actions[2].dev,		'ifb1') or
	test.assertEqual(a[44].children[0x2].actions[2].action.type,	'pipe') or
	test.assertEqual(str(a[44].children[0x2].actions[2]),		'mirror egress -> ifb1') or
	test.assertDictEqual(TcParseEmatch(a[44].children[0x2].ematch[0])[0],
		{'type': 'meta', 'lval': 446, 'op': 'eq', 'rval': ('vlan', 0, 0)}) or
	# Third one.
	test.assertEqual(a[44].children[0x3].classid,	'c:cccc') or
	test.assertEqual(a[44].children[0x3].type,	'basic') or
	test.assertEqual(a[44].children[0x3].actions[0].type,		'nat') or
	test.assertEqual(a[44].children[0x3].actions[0].direction,	'egress') or
	test.assertEqual(a[44].children[0x3].actions[0].oldipv4,	'10.144.144.0/24') or	## iproute2 has a bug there.
	test.assertEqual(a[44].children[0x3].actions[0].newipv4,	'192.168.144.0') or
	test.assertEqual(a[44].children[0x3].actions[0].action.type,	'pass') or
	test.assertEqual(str(a[44].children[0x3].actions[0]),		'nat egress 10.144.144.0/24 -> 192.168.144.0 (action pass)') or
	test.assertDictEqual(TcParseEmatch(a[44].children[0x3].ematch[0])[0],
		{'type': 'cmp', 'op': 'eq', 'align': 'u32', 'offset': 12, 'layer': 1, 'value': 0x0A0A0000, 'mask': 0xFFFF0000, 'trans': False}) or
	# Fourth one.
	test.assertEqual(a[44].children[0x4].classid,	'c:dddd') or
	test.assertEqual(a[44].children[0x4].type,	'basic') or
	test.assertEqual(a[44].children[0x4].actions[0].type,		'skbedit') or
	test.assertEqual(a[44].children[0x4].actions[0].queue_mapping, 1) or
	test.assertEqual(str(a[44].children[0x4].actions[0]),		'skbedit queue_mapping: 1') or
	test.assertDictEqual(TcParseEmatch(a[44].children[0x4].ematch[0])[0],
		{'type': 'u32', 'offset': 12, 'value': 0x0A0A0000, 'mask': 0xFFFF0000}) or
	# Fifth one.
	test.assertEqual(a[44].children[0x5].classid,	'c:eeee') or
	test.assertEqual(a[44].children[0x5].type,	'basic') or
	test.assertEqual(a[44].children[0x5].actions[0].type,		'pedit') or
	test.assertEqual(a[44].children[0x5].actions[0].keys,		[(12, 0xFFFFFFFF, 0xFFFFFFFF), (-8, 0x41410000, 0x0000FFFF)]) or
	test.assertEqual(a[44].children[0x5].actions[0].action.type,	'pass') or
	test.assertEqual(str(a[44].children[0x5].actions[0]),		'pedit keys: 2, action: pass') or
	test.assertEqual(a[44].children[0x5].actions[1].type,		'csum') or
	test.assertEqual(a[44].children[0x5].actions[1].protos,	['iph', 'tcp', 'udp']) or
	test.assertEqual(a[44].children[0x5].actions[1].action.type,	'pass') or
	test.assertEqual(str(a[44].children[0x5].actions[1]),		"csum ['iph', 'tcp', 'udp'] action: pass") or
	test.assertDictEqual(TcParseEmatch(a[44].children[0x5].ematch[0])[0],
		{'type': 'meta', 'rval': 240, 'op': 'gt', 'lval': ('nf_mark', 4, 0xFF00)}) or
	True
)

##
## FLOW, FW, ROUTE, TCINDEX FILTERS
##
## tc filter add dev enp1s0f1 protocol all pref 45 parent FFFF: handle 0xDEDA/0xFEED fw classid 0x80AA indev enp4s6 action drop
## tc filter add dev enp1s0f1 protocol all pref 46 parent FFFF: handle 0x2 flow hash keys sk-uid,sk-gid baseclass F:E123 divisor 1024 perturb 42
## tc filter add dev enp1s0f1 protocol all pref 47 parent FFFF: handle 0x2 flow map key dst addend -192.168.0.0 or 0x400 and 0xFFFE xor 1 rshift 2 divisor 256
## tc filter add dev enp1s0f1 protocol all pref 48 parent FFFF: route fromif enp4s6 to 42 classid D:BBBB
## tc filter add dev enp1s0f1 protocol all pref 49 parent FFFF: tcindex mask 0xFC shift 2 classid D:CCCC  police rate 25Mbit burst 4k
##
##
TC_FILTER_SHOW_inputs.append(b'''
filter protocol all pref 45 fw 
filter protocol all pref 45 fw handle 0xdeda/0xfeed classid :80aa input dev enp4s6 

	action order 1: gact action drop
	 random type none pass val 0
	 index 1 ref 1 bind 1
 
filter protocol all pref 46 flow 
filter protocol all pref 46 flow handle 0x2 hash keys sk-uid,sk-gid divisor 1024 baseclass f:e123 perturb 42sec 
filter protocol all pref 47 flow 
filter protocol all pref 47 flow handle 0x2 map keys dst and 0x0000fbfe or 0x00000401 rshift 2 addend 0x3f580000 divisor 256 baseclass ffff:1 
filter protocol all pref 48 route 
filter protocol all pref 48 route fh 0x8006002a flowid d:bbbb to 42 fromif enp4s6
filter protocol all pref 49 tcindex hash 64 mask 0x00fc shift 2 fall_through 
filter protocol all pref 49 tcindex handle 0x0000 classid d:cccc 
 police 0x1 rate 25Mbit burst 4Kb mtu 2Kb action reclassify overhead 0b 
ref 1 bind 1

''')
TC_FILTER_SHOW_expect.append(lambda test, a:
	test.assertEqual(a[45].actions,	[]) or
	test.assertEqual(a[45].matches,	[]) or
	test.assertEqual(a[45].handle,	None) or
	test.assertEqual(a[45].type,	'fw') or
	test.assertEqual(a[45].children[0xDEDA].classid,'0:80aa') or
	test.assertEqual(a[45].children[0xDEDA].handle,	0xDEDA) or
	test.assertEqual(a[45].children[0xDEDA].mask,	0xFEED) or
	test.assertEqual(a[45].children[0xDEDA].indev,	'enp4s6') or
	test.assertEqual(a[45].children[0xDEDA].type,	'fw') or
	test.assertEqual(a[45].children[0xDEDA].actions[0].type,	'drop') or
	test.assertEqual(str(a[45].children[0xDEDA]),	'fw handle 0xdeda/0xfeed classid 0:80aa input dev enp4s6') or

	test.assertEqual(a[46].actions,	[]) or
	test.assertEqual(a[46].matches,	[]) or
	test.assertEqual(a[46].handle,	None) or
	test.assertEqual(a[46].type,	'flow') or
	test.assertEqual(a[46].children[0x2].type,	'flow') or
	test.assertEqual(a[46].children[0x2].mode,	'hash') or
	test.assertEqual(a[46].children[0x2].keys,	['sk-uid', 'sk-gid']) or
	test.assertEqual(a[46].children[0x2].baseclass,	'f:e123') or
	test.assertEqual(a[46].children[0x2].divisor,	1024) or
	test.assertEqual(a[46].children[0x2].perturb,	42) or
	test.assertEqual(str(a[46].children[0x2]),	"handle 0x2 hash keys ['sk-uid', 'sk-gid'] divisor 1024 perturb 42sec baseclass f:e123 divisor 1024") or

	test.assertEqual(a[47].actions,	[]) or
	test.assertEqual(a[47].matches,	[]) or
	test.assertEqual(a[47].handle,	None) or
	test.assertEqual(a[47].type,	'flow') or
	test.assertEqual(a[47].children[0x2].type,	'flow') or
	test.assertEqual(a[47].children[0x2].mode,	'map') or
	test.assertEqual(a[47].children[0x2].keys,	['dst']) or
	test.assertEqual(a[47].children[0x2].baseclass,	'ffff:1') or
	test.assertEqual(a[47].children[0x2].divisor,	256) or
	test.assertEqual(a[47].children[0x2].map_and,	0xFBFE) or
	test.assertEqual(a[47].children[0x2].map_or,	0x401) or
	test.assertEqual(a[47].children[0x2].map_rshift,	2) or
	test.assertEqual(a[47].children[0x2].map_addend,	0x3F580000) or
	test.assertEqual(str(a[47].children[0x2]),	"handle 0x2 map keys ['dst'] and 0xfbfe or 0x401 addend 0x3f580000 rshift 2 baseclass ffff:1 divisor 256") or

	test.assertEqual(a[48].actions,	[]) or
	test.assertEqual(a[48].matches,	[]) or
	test.assertEqual(a[48].fh,	None) or
	test.assertEqual(a[48].type,	'route') or
	# no idea where that number came from...
	test.assertEqual(a[48].children[0x8006002A].classid,	'd:bbbb') or
	test.assertEqual(a[48].children[0x8006002A].fromif,	'enp4s6') or
	test.assertEqual(a[48].children[0x8006002A].torealm,	42) or
	test.assertEqual(a[48].children[0x8006002A].type,	'route') or
	test.assertEqual(str(a[48].children[0x8006002A]),	'route fromif enp4s6 to 42 classid d:bbbb') or

	test.assertEqual(a[49].actions,	[]) or
	test.assertEqual(a[49].matches,	[]) or
	test.assertEqual(a[49].hash,	64) or
	test.assertEqual(a[49].mask,	0xFC) or
	test.assertEqual(a[49].shift,	2) or
	test.assertEqual(a[49].fall_through,	True) or
	test.assertEqual(a[49].handle,	None) or
	test.assertEqual(a[49].type,	'tcindex') or

	test.assertEqual(a[49].children[0].classid,	'd:cccc') or
	test.assertEqual(a[49].children[0].type,	'tcindex') or
	test.assertEqual(a[49].children[0].policer.type,	'police') or
	test.assertEqual(a[49].children[0].policer.rate,	25000000) or
	test.assertEqual(a[49].children[0].policer.burst,	4000) or
	test.assertEqual(a[49].children[0].policer.mtu,		2000) or
	test.assertEqual(a[49].children[0].policer.action.type,	'reclassify') or
	test.assertEqual(str(a[49].children[0].policer),	'police rate 25000000.0 burst 4000.0 -> reclassify') or

	True
)


## ============================================================= ><8 ----------
class test_filters(unittest.TestCase):
	def setUp(self):
		self.maxDiff = 4096

	def test_all_inputs(self):
		for x in range(len(TC_FILTER_SHOW_inputs)):
			a = TcParseFilters(TC_FILTER_SHOW_inputs[x])
			self.assertTrue(TC_FILTER_SHOW_expect[x](self, a))

if __name__ == '__main__':
	unittest.main()

