import fix_path
import unittest

from parsers.tc import TcParseQdiCls

TC_QDISC_SHOW_inputs = []
TC_QDISC_SHOW_expect = []

default_pfifo_fast = {
	'qdcl':	'qdisc', 
	'type':	'pfifo_fast', 
	'rest':	b'refcnt 2 bands 3 priomap 1 2 2 2 1 2 0 0 1 1 1 1 1 1 1 1', 
	'classes':	{}, 
	'filters':	{}, 
	'qdiscs':	{}, 
}

##
## DEFAULT SETTINGS.
##
TC_QDISC_SHOW_inputs.append(b'''
qdisc pfifo_fast 0: dev enp4s6 root refcnt 2 bands 3 priomap  1 2 2 2 1 2 0 0 1 1 1 1 1 1 1 1
qdisc pfifo_fast 0: dev enp3s0 root refcnt 2 bands 3 priomap  1 2 2 2 1 2 0 0 1 1 1 1 1 1 1 1
''')

def new_qdisc(templ, i, handle='0:0'):
	q = templ.copy()
	q.update({'ifname': i, 'handle': handle})
	return {handle: q}

TC_QDISC_SHOW_expect.append({iface: new_qdisc(default_pfifo_fast, iface) for iface in ['enp3s0', 'enp4s6']})

##
## INGRESS QDISC.  (filters not parsed here)
## tc qdisc add dev enp1s0f1 ingress
## tc filter add dev enp1s0f1 parent FFFF: basic action mirred egress redirect dev ifb0
## tc qdisc add dev ifb0 root hhf
##
TC_QDISC_SHOW_inputs.append(b'''
qdisc hhf 8001: dev ifb0 root refcnt 2 [Unknown qdisc, optlen=56]
qdisc ingress ffff: dev enp1s0f1 parent ffff:fff1 ----------------
''')
TC_QDISC_SHOW_expect.append({
	'enp1s0f1': { 'ffff:0': {
		'classes': {}, 'filters': {}, 'qdiscs': {}, 'ifname': 'enp1s0f1',
		'qdcl': 'qdisc', 'type': 'ingress', 'handle': 'ffff:0',
		'rest': b'parent ffff:fff1 ----------------'
	}},
	'ifb0': { '8001:0': {
		'classes': {}, 'filters': {}, 'qdiscs': {}, 'ifname': 'ifb0',
		'qdcl': 'qdisc', 'type': 'hhf', 'handle': '8001:0',
		'rest': b'refcnt 2 [Unknown qdisc, optlen=56]'
	}},
})

class test_qdisc_pfifo_fast(unittest.TestCase):
	def setUp(self):
		self.maxDiff = 2048

	def test_all_inputs(self):
		for x in range(len(TC_QDISC_SHOW_inputs)):
			a = TcParseQdiCls(TC_QDISC_SHOW_inputs[x])
			self.assertDictEqual(a, TC_QDISC_SHOW_expect[x])

if __name__ == '__main__':
	unittest.main()

